from __future__ import unicode_literals

import urlparse

from django.contrib.sites.models import Site
from django.core import urlresolvers
from django.http import request
from django.utils import encoding
from django.utils.functional import lazy

from . import lang


__all__ = (
    "NoReverseMatch", "resolve", "reverse", "reverse_lazy",
    "simplify_redirect", "update_query",)


NoReverseMatch = urlresolvers.NoReverseMatch


def resolve(url, scheme=None, request=None, **kwargs):
    """
    Resolves the given URL by passing its path and other optional arguments to
    django.core.urlresolvers.resolve(). For fully qualified URLs, the URL
    scheme and hostname will be checked, and a ValueError will be raised upon
    mismatch. The URL scheme may be specified with the scheme argument,
    defaulting to 'http'. By default, the hostname will come from the current
    django.contrib.sites.models.Site object. Alternatively, the scheme and
    hostname may be specified by a django.http.request.HttpRequest object
    passed in the request argument.
    
    For fully qualified URLs, if no request is specified and the
    django.contrib.sites app is not installed and configured, an exception
    will be raised.
    
    For fully qualified URLs, if both a scheme and a request are specified,
    the passed scheme will take precedence over the request's scheme.
    
    """
    parts = urlparse.urlparse(url)
    if ((
        parts.scheme and
        parts.scheme != (scheme
            if scheme is not None
            else "http"
                if request is None
                else "https"
                    if request.is_secure()
                    else "http")) or (
        parts.netloc and
        parts.netloc != request.get_host()
            if request is not None
            else Site.objects.get_current().domain)):
        raise ValueError(
            "The fully qualified URL's scheme or hostname are invalid for "
            "this host.")
    return urlresolvers.resolve(parts.path)


def reverse(viewname, qualified=False, scheme=None, request=None, **kwargs):
    """
    Reverses the URL of the given view name with other optional arguments to
    django.core.urlresolvers.reverse(). If qualified is True, the URL will be
    fully qualified. The URL scheme may be specified with the scheme argument,
    defaulting to "http". By default, the hostname will come from the current
    django.contrib.sites.models.Site object. Alternatively, the scheme and
    hostname may be specified by a django.http.request.HttpRequest object
    passed in the request argument.
    
    If qualfied is True, no request is specified and the django.contrib.sites
    app is not installed and configured, an exception will be raised.
    
    If qualified is True and both a scheme and request are specified, the
    passed scheme will take precedence over the request's scheme.
    
    """
    url = urlresolvers.reverse(viewname, **kwargs)
    if qualified:
        path = "{scheme:s}://{host:s}{url:s}".format(
            scheme=(
                scheme
                    if scheme is not None
                    else "http"
                        if request is None
                        else "https"
                            if request.is_secure()
                            else "http"),
            host=(
                request.get_host()
                    if request is not None
                    else Site.objects.get_current().domain),
            url=url)
    return url


reverse_lazy = lazy(reverse, str)


def simplify_redirect(redirect, source, request=None):
    """
    Simplifies a redirect URL with respect to a source URL and returns the
    redirect URL. When both URLs' scheme and host match, the scheme and host
    are stripped from the redirect URL. When a request is specified, its
    scheme and host are also used to simplify the redirect URL.
    
    """
    redirect = urlparse.urlparse(encoding.force_str(redirect))
    source = urlparse.urlparse(encoding.force_str(source))
    empty = ("", "",)
    request = (
        (("https", request.get_host(),)
            if request.is_secure()
            else ("http", request.get_host(),))
            if request is not None
            else empty)
    if (redirect[:2] != empty and (
            redirect[:2] == source[:2] or
            (redirect[:2] == request and source[:2] == empty))):
        redirect = empty + redirect[2:]
    return urlparse.urlunparse(redirect)


def update_query(url, data):
    """
    Update a URL's query string with additional data. The data may be a
    dictionary or an instance of django.utils.datastructures.MultiValueDict.
    The URL should be a string or a lazy string.
    
    """
    parts = urlparse.urlparse(encoding.force_str(url))
    return urlparse.urlunparse(
        parts[:4] +
        (lang.updated(
            request.QueryDict(parts.query, mutable=True),
            data or {}).urlencode(safe="/"),) +
        parts[5:])



from . import base, dates, detail, edit, list
from .base import View, TemplateView, RedirectView
from .dates import (ArchiveIndexView, YearArchiveView, MonthArchiveView,
    WeekArchiveView, DayArchiveView, TodayArchiveView, DateDetailView,)
from .detail import DetailView
from .edit import FormView, CreateView, UpdateView, DeleteView
from .list import ListView


__all__ = (
    "View", "TemplateView", "RedirectView", "ArchiveIndexView",
    "YearArchiveView", "MonthArchiveView", "WeekArchiveView", "DayArchiveView",
    "TodayArchiveView", "DateDetailView", "DetailView", "FormView",
    "CreateView", "UpdateView", "DeleteView", "ListView",)



from __future__ import unicode_literals

import datetime

from daydreamer.views import generic
from daydreamer.views.behaviors import http

from . import base


class RequireGETTestCase(base.TestCase):
    """
    Tests for the RequireGET view behavior.
    
    """
    view_classes = (http.RequireGET, generic.View,)
    
    def test_get_allowed(self):
        """
        Check that GET requests are allowed.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"get": content},
            status_code=200,
            content=content)
    
    def test_head_denied(self):
        """
        Check that HEAD requests are denied.
        
        """
        self.assertViewBehavior(
            {"head": self.unique()},
            method="head",
            status_code=405)
    
    def test_options_denied(self):
        """
        Check that OPTIONS requests are denied.
        
        """
        self.assertViewBehavior(
            {"options": self.unique()},
            method="options",
            status_code=405)
    
    def test_post_denied(self):
        """
        Check that POST requests are denied.
        
        """
        self.assertViewBehavior(
            {"post": self.unique()},
            method="post",
            status_code=405)
    
    def test_put_denied(self):
        """
        Check that PUT requests are denied.
        
        """
        self.assertViewBehavior(
            {"put": self.unique()},
            method="put",
            status_code=405)
    
    def test_delete_denied(self):
        """
        Check that DELETE requests are denied.
        
        """
        self.assertViewBehavior(
            {"delete": self.unique()},
            method="delete",
            status_code=405)


class RequirePOSTTestCase(base.TestCase):
    """
    Tests for the RequirePOST view behavior.
    
    """
    view_classes = (http.RequirePOST, generic.View,)
    
    def test_post_allowed(self):
        """
        Check that POST requests are allowed.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"post": content},
            method="post",
            status_code=200,
            content=content)
    
    def test_get_allowed(self):
        """
        Check that GET requests are allowed.
        
        """
        self.assertViewBehavior(
            {"get": self.unique()},
            status_code=405)
    
    def test_head_denied(self):
        """
        Check that HEAD requests are denied.
        
        """
        self.assertViewBehavior(
            {"head": self.unique()},
            method="head",
            status_code=405)
    
    def test_options_denied(self):
        """
        Check that OPTIONS requests are denied.
        
        """
        self.assertViewBehavior(
            {"options": self.unique()},
            method="options",
            status_code=405)
    
    def test_put_denied(self):
        """
        Check that PUT requests are denied.
        
        """
        self.assertViewBehavior(
            {"put": self.unique()},
            method="put",
            status_code=405)
    
    def test_delete_denied(self):
        """
        Check that DELETE requests are denied.
        
        """
        self.assertViewBehavior(
            {"delete": self.unique()},
            method="delete",
            status_code=405)


class RequireSafeTestCase(base.TestCase):
    """
    Tests for the RequireSafe view behavior.
    
    """
    view_classes = (http.RequireSafe, generic.View,)
    
    def test_get_allowed(self):
        """
        Check that GET requests are allowed.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"get": content},
            status_code=200,
            content=content)
    
    def test_head_denied(self):
        """
        Check that HEAD requests are allowed.
        
        """
        self.assertViewBehavior(
            {"head": self.unique()},
            method="head",
            status_code=200,
            content="")
    
    def test_options_denied(self):
        """
        Check that OPTIONS requests are denied.
        
        """
        self.assertViewBehavior(
            {"options": self.unique()},
            method="options",
            status_code=405)
    
    def test_post_denied(self):
        """
        Check that POST requests are denied.
        
        """
        self.assertViewBehavior(
            {"post": self.unique()},
            method="post",
            status_code=405)
    
    def test_put_denied(self):
        """
        Check that PUT requests are denied.
        
        """
        self.assertViewBehavior(
            {"put": self.unique()},
            method="put",
            status_code=405)
    
    def test_delete_denied(self):
        """
        Check that DELETE requests are denied.
        
        """
        self.assertViewBehavior(
            {"delete": self.unique()},
            method="delete",
            status_code=405)


class ConditionTestCase(base.TestCase):
    """
    Tests for the Condition view behavior.
    
    Note that this does not confirm all possible ETag and Last-Modified
    behaviors. It just shows that the decorator is wired in correctly.
    
    """
    view_classes = (http.Condition, generic.View,)
    
    def test_etag(self):
        """
        Check that the ETag header is set on the response.
        
        """
        etag = self.unique()
        content = self.unique()
        def condition_etag(self, request, *args, **kwargs):
            return etag
        self.assertViewBehavior(
            {"condition_etag": condition_etag, "get": content},
            status_code=200,
            content=content,
            headers_exact={"ETag": self.format_etag(etag)})
    
    def test_etag_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        when no valid ETag header is sent and that an ETag header is not set on
        the response.
        
        """
        etag = self.unique()
        def condition_etag(self, request, *args, **kwargs):
            return etag
        self.assertViewBehavior(
            {"condition_etag": condition_etag},
            status_code=405,
            headers_exclude="ETag")
    
    def test_etag_match_not_modified(self):
        """
        Check for a not modified response on ETag match.
        
        """
        etag = self.unique()
        def condition_etag(self, request, *args, **kwargs):
            return etag
        self.assertViewBehavior(
            {"condition_etag": condition_etag, "get": self.unique()},
            headers={"HTTP_IF_NONE_MATCH": self.format_etag(etag)},
            status_code=304,
            content="",
            headers_exact={"ETag": self.format_etag(etag)})
    
    def test_etag_match_precdence(self):
        """
        Check that the default HTTP method name protection takes precedence
        upon ETag match and than an ETag header is not set on the response.
        
        """
        etag = self.unique()
        def condition_etag(self, request, *args, **kwargs):
            return etag
        self.assertViewBehavior(
            {"condition_etag": condition_etag},
            headers={"HTTP_IF_NONE_MATCH": self.format_etag(etag)},
            status_code=405,
            headers_exclude="ETag")
    
    def test_etag_fail(self):
        """
        Check for a precondition fail response for an ETag mismatch.
        
        """
        etag = self.unique()
        def condition_etag(self, request, *args, **kwargs):
            return etag
        self.assertViewBehavior(
            {"condition_etag": condition_etag, "get": self.unique()},
            headers={"HTTP_IF_MATCH": self.format_etag(self.unique())},
            status_code=412,
            headers_exact={"ETag": self.format_etag(etag)})
    
    def test_etag_fail_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        upon ETag mismatch and that an ETag header is not set on the response.
        
        """
        etag = self.unique()
        def condition_etag(self, request, *args, **kwargs):
            return etag
        self.assertViewBehavior(
            {"condition_etag": condition_etag},
            headers={"HTTP_IF_MATCH": self.format_etag(self.unique())},
            status_code=405,
            headers_exclude="ETag")
    
    def test_etag_miss(self):
        """
        Check the ETag header is updated upon miss.
        
        """
        etag = self.unique()
        content = self.unique()
        def condition_etag(self, request, *args, **kwargs):
            return etag
        self.assertViewBehavior(
            {"condition_etag": condition_etag, "get": content},
            headers={"HTTP_IF_NONE_MATCH": self.format_etag(self.unique())},
            status_code=200,
            content=content,
            headers_exact={"ETag": self.format_etag(etag)})
    
    def test_etag_miss_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        upon ETag header miss and that the ETag header is not set on
        the response.
        
        """
        etag = self.unique()
        def condition_etag(self, request, *args, **kwargs):
            return etag
        self.assertViewBehavior(
            {"condition_etag": condition_etag},
            headers={"HTTP_IF_NONE_MATCH": self.format_etag(self.unique())},
            status_code=405,
            headers_exclude="ETag")
    
    def test_last_modified(self):
        """
        Check that the last modified header is set.
        
        """
        last_modified = datetime.datetime.now()
        content = self.unique()
        def condition_last_modified(self, request, *args, **kwargs):
            return last_modified
        self.assertViewBehavior({
                "condition_last_modified": condition_last_modified,
                "get": content},
            status_code=200,
            content=content,
            headers_exact={
                "Last-Modified": self.format_datetime(last_modified)})
    
    def test_last_modified_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        when no valid last modified header is sent and that a last modified
        header is not set on the response.
        
        """
        last_modified = datetime.datetime.now()
        def condition_last_modified(self, request, *args, **kwargs):
            return last_modified
        self.assertViewBehavior(
            {"condition_last_modified": condition_last_modified},
            status_code=405,
            headers_exclude="Last-Modified")
    
    def test_last_modified_match_not_modified(self):
        """
        Check for a not modified response on last modified match.
        
        """
        last_modified = datetime.datetime.now()
        def condition_last_modified(self, request, *args, **kwargs):
            return last_modified
        self.assertViewBehavior({
                "condition_last_modified": condition_last_modified,
                "get": self.unique()},
            headers={
                "HTTP_IF_MODIFIED_SINCE":
                    self.format_datetime(
                        last_modified + datetime.timedelta(hours=1))},
            status_code=304,
            headers_exact={
                "Last-Modified": self.format_datetime(last_modified)})
    
    def test_last_modified_match_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        upon last modified match and that a last modified header is not set on
        the response.
        
        """
        last_modified = datetime.datetime.now()
        def condition_last_modified(self, request, *args, **kwargs):
            return last_modified
        self.assertViewBehavior(
            {"condition_last_modified": condition_last_modified},
            headers={
                "HTTP_IF_MODIFIED_SINCE":
                    self.format_datetime(
                        last_modified + datetime.timedelta(hours=1))},
            status_code=405,
            headers_exclude="Last-Modified")
    
    def test_last_modified_miss(self):
        """
        Check that the last modified header is updated upon miss.
        
        """
        last_modified = datetime.datetime.now()
        content = self.unique()
        def condition_last_modified(self, request, *args, **kwargs):
            return last_modified + datetime.timedelta(hours=1)
        self.assertViewBehavior({
                "condition_last_modified": condition_last_modified,
                "get": content},
            headers={
                "HTTP_IF_MODIFIED_SINCE": self.format_datetime(last_modified)},
            status_code=200,
            content=content,
            headers_exact={
                "Last-Modified":
                    self.format_datetime(
                        last_modified + datetime.timedelta(hours=1))})
    
    def test_last_modified_miss_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        upon last modified header miss and that a last modified header is
        not set on the response.
        
        """
        last_modified = datetime.datetime.now()
        def condition_last_modified(self, request, *args, **kwargs):
            return last_modified + datetime.timedelta(hours=1)
        self.assertViewBehavior(
            {"condition_last_modified": condition_last_modified},
            headers={
                "HTTP_IF_MODIFIED_SINCE": self.format_datetime(last_modified)},
            status_code=405,
            headers_exclude="Last-Modified")



from __future__ import unicode_literals

from daydreamer.core import lang

from .. import testcases
from . import redirects, views


class TestCase(views.TestCase, redirects.TestCase, testcases.TestCase):
    """
    A test case that provides a variety of general testing features and an
    enhanced version of the base class' assertViewBehavior().
    
    """
    # Utilities.
    def unique_path(self):
        """
        Returns a unique path for use in testing views.
        
        While operating outside of the URL resolution framework, this should be
        used to generate a path to pass to the client to verify that a view's
        behavior is independent of the request's path.
        
        """
        return "/{unique:s}/".format(unique=self.unique())
    
    # Assertions.
    def assertViewBehavior(self, view, view_args=None, view_kwargs=None,
            path=None, method="get", method_args=None, method_kwargs=None,
            view_includes=None, view_excludes=None, view_exact=None,
            **kwargs):
        """
        Sends an HTTP request to the given view with arguments and keyword
        arguments using the client's method.
        
        If method_args or method_kwargs is specified, their values will
        take precedence to allow for cooperative super() calls.
        
        If the path is not specified, a unique path will be generated.
        
        If view_includes is specified, the generated view function must have the
        specified attributes after it has been called. If view_excludes is
        specified, the generated view function must not have the specified
        attributes after the view has been called. If view_exact is specified,
        the generated view function must have the specified names and exact
        attribute values after the view has been called.
        
        See daydreamer.test.testcases.TestCase.assertViewBehavior for
        documentation of the other arguments.
        
        """
        # Use a unique path?
        path = self.unique_path() if path is None else path
        
        # Generate or use the passed method arguments and keyword arguments?
        if method_args is None or method_kwargs is None:
            method_args = (view,)
            method_kwargs = {
                "view_args": view_args,
                "view_kwargs": view_kwargs,
                "path": path}
        
        # Set up the assertions.
        method_assertions = lang.updated(
            kwargs.pop("method_assertions", {}),
            dict(
                view_includes=view_includes,
                view_excludes=view_excludes,
                view_exact=view_exact),
            copy=True)
        
        return super(TestCase, self).assertViewBehavior(
            None, method=method, method_args=method_args,
            method_kwargs=method_kwargs,
            method_assertions=method_assertions,
            **kwargs)



from __future__ import unicode_literals

from django.conf import settings
from django.middleware import csrf

from daydreamer.test.views import client


class Client(client.Client):
    """
    A specialized test client that can set a CSRF cookie.
    
    """
    def set_csrf_cookie(self):
        """
        Sets a unique CSRF cookie for subsequent requests to use. Returns
        the new cookie's value.
        
        """
        self.cookies[settings.CSRF_COOKIE_NAME] = csrf._get_new_csrf_key()
        self.cookies[settings.CSRF_COOKIE_NAME].update({
            "max_age": 60 * 60 * 24 * 7 * 52,
            "domain": settings.CSRF_COOKIE_DOMAIN,
            "path": settings.CSRF_COOKIE_PATH,
            "secure": settings.CSRF_COOKIE_SECURE,
            "httponly": settings.CSRF_COOKIE_HTTPONLY})
        return self.cookies.get(settings.CSRF_COOKIE_NAME).value
    
    def get_csrf_cookie(self, set_missing=True):
        """
        Returns the current CSRF cookie value. If set_missing is True,
        sets and returns the CSRF cookie value when it's missing.
        
        """
        cookie = self.cookies.get(settings.CSRF_COOKIE_NAME)
        if cookie:
            return cookie.value
        elif set_missing:
            return self.set_csrf_cookie()
        return None



from __future__ import unicode_literals

import calendar
import datetime

from django.utils import http as http_utils

from daydreamer.tests.views.core import http


class TestCase(http.TestCase):
    """
    Common utilities for testing HTTP view behaviors.
    
    """
    # Utilities.
    def format_etag(self, etag):
        """
        Quote the given ETag for use in an HTTP header.
        
        """
        return http_utils.quote_etag(etag)
    
    def format_datetime(self, dt):
        """
        Format a datetime for use in an HTTP header.
        
        """
        return http_utils.http_date(calendar.timegm(dt.utctimetuple()))



from __future__ import unicode_literals

from .. import base
from . import client


__all__ = ("TestCase",)


class TestCase(base.TestCase):
    """
    A test case for testing views without using the URL resolution framework
    by employing a customized test client and request handler.
    
    See daydreamer.test.views.client and daydreamer.test.views.handler
    for implementation details.
    
    """
    client_class = client.Client



from __future__ import unicode_literals

from django.conf import settings
from django.contrib import auth, messages
from django.contrib.auth import models as auth_models
from django.contrib.contenttypes import models as type_models
from django.utils import six

from .. import base


class TestCase(base.TestCase):
    """
    Common utilities for testing Denial view behaviors. This is an abstract
    base class that subclasses of Denial should use to test edge cases and
    configuration coupling in the denial framework.
    
    Specify a prefix to use for the attributes passed to self.view().
    
    You must also provide overrides for the four setup_*() methods.
    
    """
    prefix = None
    
    # Test environment setups.
    def setup_unauth_fail(self):
        """
        Sets up an unauthenticated user and returns view attributes that should
        cause the user to fail the test.
        
        Subclasses must implement this.
        
        """
        raise NotImplementedError
    
    def setup_auth_fail(self):
        """
        Sets up an authenticated user and returns view attributes that should
        cause the user to fail the test.
        
        Subclasses must implement this.
        
        """
        raise NotImplementedError
    
    def setup_unauth_pass(self):
        """
        Sets up an unauthenticated user and returns view attributes that should
        cause the user to pass the test.
        
        Subclasses must implement this.
        
        """
        raise NotImplementedError
    
    def setup_auth_pass(self):
        """
        Sets up an authenticated user and returns view attributes that should
        cause the user to pass the test.
        
        Subclasses must implement this.
        
        """
        raise NotImplementedError
    
    # Utilities.
    def unique_username(self):
        """
        Create a unique string appropriate for a username.
        
        """
        return self.unique()[:30]
    
    def unique_group(self):
        """
        Create a unique string appropriate for a group.
        
        """
        return self.unique()[:80]
    
    def unique_permission(self):
        """
        Create a unique string appropriate for a permission.
        
        """
        return self.unique()[:100]
    
    def create_authenticated_user(self, **attrs):
        """
        Creates a user with a unique username and password, sets the specified
        attributes on the user and logs in. Returns the new,
        authenticated user.
        
        """
        username = self.unique_username()
        password = self.unique()
        user = auth.get_user_model().objects.create_user(username, password=password)
        if attrs:
            for name, value in six.iteritems(attrs):
                setattr(user, name, value)
            user.save()
        self.client.login(username=username, password=password)
        return user
    
    def create_group(self):
        """
        Creates a group with a unique name and returns it.
        
        """
        return auth_models.Group.objects.create(name=self.unique_group())
    
    def create_permission(self):
        """
        Creates a permission on the User model with a unique codename and
        returns it.
        
        """
        return auth_models.Permission.objects.create(
            content_type=type_models.ContentType.objects.get_for_model(
                auth.get_user_model()),
            codename=self.unique_permission())
    
    def view(self, **attrs):
        """
        Adds self.prefix to all of the attributes, except for those
        in self.generate_http_verbs.
        
        """
        return super(TestCase, self).view(**{
            key if key in self.generate_http_verbs
                else "_".join((self.prefix, key))
                    if key
                    else self.prefix: value
            for key, value in six.iteritems(attrs)})
    
    # Common tests.
    def test_unauth_fail(self, setup_unauth_fail=None):
        """
        Check default behavior for an unauthenticated user failing the test.
        
        Settings:
            (defaults)
        
        A custom setup_unauth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        self.assertViewBehavior(
            {"get": self.unique()},
            setup_unauth_fail or self.setup_unauth_fail,
            path=path,
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_unauth_fail_raise(self, setup_unauth_fail=None):
        """
        Check exception raising behavior for an unauthenticated user failing
        the test.
        
        Settings:
            <prefix>_raise=True
        
        A custom setup_unauth_fail callback may be provided to reuse this
        test method.
        
        """
        self.assertViewBehavior(
            {"raise": True, "get": self.unique()},
            setup_unauth_fail or self.setup_unauth_fail,
            status_code=403)
    
    def test_unauth_fail_raise_exception(self, setup_unauth_fail=None):
        """
        Check custom exception exception raising behavior for an
        unauthenticated user failing the test.
        
        Settings:
            <prefix>_raise=True
            <prefix>_exception=<unique>
        
        A custom setup_unauth_fail callback may be provided to reuse this
        test method.
        
        """
        class TestException(Exception):
            pass
        exception = TestException(self.unique())
        self.assertViewBehavior(
            {"raise": True, "exception": exception, "get": self.unique()},
            setup_unauth_fail or self.setup_unauth_fail,
            exception=exception)
    
    def test_unauth_fail_exception(self, setup_unauth_fail=None):
        """
        Check that exception raising behavior is controlled by <prefix>_raise,
        not by a coupling error with <prefix>_exception for an unauthenticated
        user failing the test.
        
        Settings:
            <prefix>_exception=<unique>
        
        A custom setup_unauth_fail callback may be provided to reuse this
        test method.
        
        """
        class TestException(Exception):
            pass
        path = self.unique_path()
        exception = TestException(self.unique())
        self.assertViewBehavior(
            {"exception": exception, "get": self.unique()},
            setup_unauth_fail or self.setup_unauth_fail,
            path=path,
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_unauth_fail_message(self, setup_unauth_fail=None):
        """
        Check message enqueuing behavior for an unauthenticated user failing
        the test.
        
        Settings:
            <prefix>_message=<unique>
        
        A custom setup_unauth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        message = self.unique()
        self.assertViewBehavior(
            {"message": message, "get": self.unique()},
            setup_unauth_fail or self.setup_unauth_fail,
            path=path,
            follow=True,
            message=message,
            message_level=messages.WARNING,
            message_tags="warning",
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_unauth_fail_message_message_level(self, setup_unauth_fail=None):
        """
        Check message enqueuing behavior with a custom message level for an
        unauthenticated user failing the test.
        
        Settings:
            <prefix>_message=<unique>
            <prefix>_message_level=messages.SUCCESS
        
        A custom setup_unauth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        message = self.unique()
        level = messages.SUCCESS
        self.assertViewBehavior(
            {"message": message, "message_level": level, "get": self.unique()},
            setup_unauth_fail or self.setup_unauth_fail,
            path=path,
            follow=True,
            message=message,
            message_level=level,
            message_tags="success",
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_unauth_fail_message_message_tags(self, setup_unauth_fail=None):
        """
        Check message enqueuing behavior with custom message tags for an
        unauthenticated user failing the test.
        
        Settings:
            <prefix>_message=<unique>
            <prefix>_message_tags=<unique>
        
        A custom setup_unauth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        message = self.unique()
        tags = " ".join((self.unique(), self.unique()))
        self.assertViewBehavior(
            {"message": message, "message_tags": tags, "get": self.unique()},
            setup_unauth_fail or self.setup_unauth_fail,
            path=path,
            follow=True,
            message=message,
            message_level=messages.WARNING,
            message_tags=" ".join(("warning", tags,)),
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_unauth_fail_message_level(self, setup_unauth_fail=None):
        """
        Check that message enqueuing behavior is controlled by
        <prefix>_message, not by a coupling error with <prefix>_message_level
        for an unauthenticated user failing the test.
        
        Settings:
            <prefix>_message_level=<unique>
        
        A custom setup_unauth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        self.assertViewBehavior(
            {"message_level": messages.SUCCESS, "get": self.unique()},
            setup_unauth_fail or self.setup_unauth_fail,
            path=path,
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_unauth_fail_message_tags(self, setup_unauth_fail=None):
        """
        Check that message enqueuing behavior is controlled by
        <prefix>_message, not by a coupling error with <prefix>_message_tags
        for an unauthenticated user failing the test.
        
        A custom setup_unauth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        tags = " ".join((self.unique(), self.unique(),))
        self.assertViewBehavior(
            {"message_tags": tags, "get": self.unique()},
            setup_unauth_fail or self.setup_unauth_fail,
            path=path,
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)

    def test_unauth_fail_redirect_url(self, setup_unauth_fail=None):
        """
        Check redirect behavior to a custom URL for an unauthenticated user
        failing the test.
        
        Settings:
            <prefix>_redirect_url=<unique>
        
        A custom setup_unauth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        redirect = self.unique_path()
        self.assertViewBehavior(
            {"redirect_url": redirect, "get": self.unique()},
            setup_unauth_fail or self.setup_unauth_fail,
            path=path,
            redirect_url=redirect,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_unauth_fail_redirect_next_url(self, setup_unauth_fail=None):
        """
        Check redirect behavior with a custom next URL query parameter value
        for an unauthenticated user failing the test.
        
        Settings:
            <prefix>_redirect_next_url=<unique>
        
        A custom setup_unauth_fail callback may be provided to reuse this
        test method.
        
        """
        next = self.unique_path()
        self.assertViewBehavior(
            {"redirect_next_url": next, "get": self.unique()},
            setup_unauth_fail or self.setup_unauth_fail,
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=next,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_unauth_fail_redirect_next_name(self, setup_unauth_fail=None):
        """
        Check redirect behavior with a custom next URL query parameter name
        for an unauthenticated user failing the test.
        
        Settings:
            <prefix>_redirect_next_name=<unique>
        
        A custom setup_unauth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        name = self.unique()
        self.assertViewBehavior(
            {"redirect_next_name": name, "get": self.unique()},
            setup_unauth_fail or self.setup_unauth_fail,
            path=path,
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=name)
    
    def test_unauth_fail_no_redirect_next_name(self, setup_unauth_fail=None):
        """
        Check redirect behavior with no return query parameter name for an
        unauthenticated user failing the test.
        
        Settings:
            <prefix>_redirect_next_name=None
        
        A custom setup_unauth_fail callback may be provided to reuse this
        test method.
        
        """
        self.assertViewBehavior(
            {"redirect_next_name": None, "get": self.unique()},
            setup_unauth_fail or self.setup_unauth_fail,
            redirect_url=settings.LOGIN_URL)
    
    def test_unauth_fail_precedence(self, setup_unauth_fail=None):
        """
        Check that the view behavior's behavior takes precedence when
        using an unsupported HTTP method for an unauthenticated user failing
        the test.
        
        Settings:
            (defaults)
        
        A custom setup_unauth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        self.assertViewBehavior(
            {"get": self.unique()},
            setup_unauth_fail or self.setup_unauth_fail,
            path=path,
            method="post",
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_auth_fail(self, setup_auth_fail=None):
        """
        Check default behavior for an authenticated user failing the test.
        
        Settings:
            (defaults)
        
        A custom setup_auth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        self.assertViewBehavior(
            {"get": self.unique()},
            setup_auth_fail or self.setup_auth_fail,
            path=path,
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_auth_fail_raise(self, setup_auth_fail=None):
        """
        Check exception raising behavior for an authenticated user failing
        the test.
        
        Settings:
            <prefix>_raise=True
        
        A custom setup_auth_fail callback may be provided to reuse this
        test method.
        
        """
        self.assertViewBehavior(
            {"raise": True, "get": self.unique()},
            setup_auth_fail or self.setup_auth_fail,
            status_code=403)
    
    def test_auth_fail_raise_exception(self, setup_auth_fail=None):
        """
        Check custom exception exception raising behavior for an
        authenticated user failing the test.
        
        Settings:
            <prefix>_raise=True
            <prefix>_exception=<unique>
        
        A custom setup_auth_fail callback may be provided to reuse this
        test method.
        
        """
        class TestException(Exception):
            pass
        exception = TestException(self.unique())
        self.assertViewBehavior(
            {"raise": True, "exception": exception, "get": self.unique()},
            setup_auth_fail or self.setup_auth_fail,
            exception=exception)
    
    def test_auth_fail_exception(self, setup_auth_fail=None):
        """
        Check that exception raising behavior is controlled by <prefix>_raise,
        not by a coupling error with <prefix>_exception for an authenticated
        user failing the test.
        
        Settings:
            <prefix>_exception=<unique>
        
        A custom setup_auth_fail callback may be provided to reuse this
        test method.
        
        """
        class TestException(Exception):
            pass
        path = self.unique_path()
        exception = TestException(self.unique())
        self.assertViewBehavior(
            {"exception": exception, "get": self.unique()},
            setup_auth_fail or self.setup_auth_fail,
            path=path,
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_auth_fail_message(self, setup_auth_fail=None):
        """
        Check message enqueuing behavior for an authenticated user failing
        the test.
        
        Settings:
            <prefix>_message=<unique>
        
        A custom setup_auth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        message = self.unique()
        self.assertViewBehavior(
            {"message": message, "get": self.unique()},
            setup_auth_fail or self.setup_auth_fail,
            path=path,
            follow=True,
            message=message,
            message_level=messages.WARNING,
            message_tags="warning",
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_auth_fail_message_message_level(self, setup_auth_fail=None):
        """
        Check message enqueuing behavior with a custom message level for an
        authenticated user failing the test.
        
        Settings:
            <prefix>_message=<unique>
            <prefix>_message_level=messages.SUCCESS
        
        A custom setup_auth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        message = self.unique()
        level = messages.SUCCESS
        self.assertViewBehavior(
            {"message": message, "message_level": level, "get": self.unique()},
            setup_auth_fail or self.setup_auth_fail,
            path=path,
            follow=True,
            message=message,
            message_level=level,
            message_tags="success",
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_auth_fail_message_message_tags(self, setup_auth_fail=None):
        """
        Check message enqueuing behavior with custom message tags for an
        authenticated user failing the test.
        
        Settings:
            <prefix>_message=<unique>
            <prefix>_message_tags=<unique>
        
        A custom setup_auth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        message = self.unique()
        tags = " ".join((self.unique(), self.unique(),))
        self.assertViewBehavior(
            {"message": message, "message_tags": tags, "get": self.unique()},
            setup_auth_fail or self.setup_auth_fail,
            path=path,
            follow=True,
            message=message,
            message_level=messages.WARNING,
            message_tags=" ".join(("warning", tags,)),
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_auth_fail_message_level(self, setup_auth_fail=None):
        """
        Check that message enqueuing behavior is controlled by
        <prefix>_message, not by a coupling error with <prefix>_message_level
        for an authenticated user failing the test.
        
        Settings:
            <prefix>_message_level=<unique>
        
        A custom setup_auth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        self.assertViewBehavior(
            {"message_level": messages.SUCCESS, "get": self.unique()},
            setup_auth_fail or self.setup_auth_fail,
            path=path,
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_auth_fail_message_tags(self, setup_auth_fail=None):
        """
        Check that message enqueuing behavior is controlled by
        <prefix>_message, not by a coupling error with <prefix>_message_tags
        for an authenticated user failing the test.
        
        A custom setup_auth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        tags = " ".join((self.unique(), self.unique(),))
        self.assertViewBehavior(
            {"message_tags": tags, "get": self.unique()},
            setup_auth_fail or self.setup_auth_fail,
            path=path,
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_auth_fail_redirect_url(self, setup_auth_fail=None):
        """
        Check redirect behavior to a custom URL for an authenticated user
        failing the test.
        
        Settings:
            <prefix>_redirect_url=<unique>
        
        A custom setup_auth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        redirect = self.unique_path()
        self.assertViewBehavior(
            {"redirect_url": redirect, "get": self.unique()},
            setup_auth_fail or self.setup_auth_fail,
            path=path,
            redirect_url=redirect,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_auth_fail_redirect_next_url(self, setup_auth_fail=None):
        """
        Check redirect behavior with a custom next URL query parameter value
        for an authenticated user failing the test.
        
        Settings:
            <prefix>_redirect_next_url=<unique>
        
        A custom setup_auth_fail callback may be provided to reuse this
        test method.
        
        """
        next = self.unique_path()
        self.assertViewBehavior(
            {"redirect_next_url": next, "get": self.unique()},
            setup_auth_fail or self.setup_auth_fail,
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=next,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_auth_fail_redirect_next_name(self, setup_auth_fail=None):
        """
        Check redirect behavior with a custom next URL query parameter name
        for an authenticated user failing the test.
        
        Settings:
            <prefix>_redirect_next_name=<unique>
        
        A custom setup_auth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        name = self.unique()
        self.assertViewBehavior(
            {"redirect_next_name": name, "get": self.unique()},
            setup_auth_fail or self.setup_auth_fail,
            path=path,
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=name)
    
    def test_auth_fail_no_redirect_next_name(self, setup_auth_fail=None):
        """
        Check redirect behavior with no return query parameter name for an
        authenticated user failing the test.
        
        Settings:
            <prefix>_redirect_next_name=None
        
        A custom setup_auth_fail callback may be provided to reuse this
        test method.
        
        """
        self.assertViewBehavior(
            {"redirect_next_name": None, "get": self.unique()},
            setup_auth_fail or self.setup_auth_fail,
            redirect_url=settings.LOGIN_URL)
    
    def test_auth_fail_precedence(self, setup_auth_fail=None):
        """
        Check that the view behavior's behavior takes precedence when
        using an unsupported HTTP method for an authenticated user failing
        the test.
        
        Settings:
            (defaults)
        
        A custom setup_auth_fail callback may be provided to reuse this
        test method.
        
        """
        path = self.unique_path()
        self.assertViewBehavior(
            setup=setup_auth_fail or self.setup_auth_fail,
            path=path,
            method="post",
            redirect_url=settings.LOGIN_URL,
            redirect_next_url=path,
            redirect_next_name=auth.REDIRECT_FIELD_NAME)
    
    def test_unauth_pass(self, setup_unauth_pass=None):
        """
        Check default behavior for an unauthenticated user passing the test.
        
        Settings:
            (defaults)
        
        A custom setup_unauth_pass callback may be provided to reuse this
        test method.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"get": content},
            setup_unauth_pass or self.setup_unauth_pass,
            status_code=200,
            content=content)
    
    def test_unauth_pass_raise(self, setup_unauth_pass=None):
        """
        Check that <prefix>_raise is ignored for an unauthenticated user
        passing the test.
        
        Settings:
            <prefix>_raise=True
        
        A custom setup_unauth_pass callback may be provided to reuse this
        test method.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"raise": True, "get": content},
            setup_unauth_pass or self.setup_unauth_pass,
            status_code=200,
            content=content)
    
    def test_unauth_pass_message(self, setup_unauth_pass=None):
        """
        Check that <prefix>_message is ignored for an unauthenticated user
        passing the test.
        
        Settings:
            <prefix>_message=<unique>
        
        A custom setup_unauth_pass callback may be provided to reuse this
        test method.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"message": self.unique(), "get": content},
            setup_unauth_pass or self.setup_unauth_pass,
            status_code=200,
            content=content)
    
    def test_auth_pass_message(self, setup_auth_pass=None):
        """
        Check that <prefix>_message is ignored for an authenticated user
        passing the test.
        
        Settings:
            <prefix>_message=<unique>
        
        A custom setup_auth_pass callback may be provided to reuse this
        test method.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"message": self.unique(), "get": content},
            setup_auth_pass or self.setup_auth_pass,
            status_code=200,
            content=content)
    
    def test_auth_pass(self, setup_auth_pass=None):
        """
        Check default behavior for an authenticated user passing the test.
        
        Settings:
            (defaults)
        
        A custom setup_auth_pass callback may be provided to reuse this
        test method.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"get": content},
            setup_auth_pass or self.setup_auth_pass,
            status_code=200,
            content=content)
    
    def test_auth_pass_raise(self, setup_auth_pass=None):
        """
        Check that <prefix>_raise is ignored for an authenticated user
        passing the test.
        
        Settings:
            <prefix>_raise=True
        
        A custom setup_auth_pass callback may be provided to reuse this
        test method.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"raise": True, "get": content},
            setup_auth_pass or self.setup_auth_pass,
            status_code=200,
            content=content)
    
    def test_unauth_pass_precedence(self, setup_unauth_pass=None):
        """
        Check that the behavior of the base view class takes precedence
        when using an unsupported HTTP method for an unauthenticated user
        passing the test.
        
        Settings:
            (defaults)
        
        A custom setup_unauth_pass callback may be provided to reuse this
        test method.
        
        """
        self.assertViewBehavior(
            setup=setup_unauth_pass or self.setup_unauth_pass,
            status_code=405)
    
    def test_auth_pass_precedence(self, setup_auth_pass=None):
        """
        Check that the behavior of the base view class takes precedence
        when using an unsupported HTTP method for an authenticated user
        passing the test.
        
        Settings:
            (defaults)
        
        A custom setup_auth_pass callback may be provided to reuse this
        test method.
        
        """
        self.assertViewBehavior(
            setup=setup_auth_pass or self.setup_auth_pass,
            status_code=405)



from __future__ import unicode_literals

from django.views import generic

from .base import View


__all__ = ("ListView",)


class ListView(generic.ListView, View):
    """
    Extends Django's ListView class with features from
    daydreamer.views.generic.View.
    
    """
    pass



from __future__ import unicode_literals

from daydreamer.views.behaviors import debug

from . import base


class SensitiveVariablesTestCase(base.TestCase):
    """
    Tests for the SensitiveVariables view behavior.
    
    """
    view_classes = debug.SensitiveVariables
    
    def test_defaults(self):
        """
        Check that the special marker for all sensitive variables is set on
        the generated view.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"get": content},
            status_code=200,
            content=content,
            view_exact={"sensitive_variables": "__ALL__"})
    
    def test_single(self):
        """
        Check that a single sensitive variable is set on the
        generated view.
        
        """
        content = self.unique()
        variable = self.unique_variable()
        self.assertViewBehavior(
            {"sensitive_variables": variable, "get": content},
            status_code=200,
            content=content,
            view_exact={"sensitive_variables": (variable,)})
    
    def test_multiple(self):
        """
        Check that multiple sensitive variables are set on the
        generated view.
        
        """
        content = self.unique()
        variables = (self.unique_variable(), self.unique_variable(),)
        self.assertViewBehavior(
            {"sensitive_variables": variables, "get": content},
            status_code=200,
            content=content,
            view_exact={"sensitive_variables": variables})
    
    def test_disabled(self):
        """
        Check that sensitive variables are not set on the generated view
        when the behavior is disabled.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"sensitive_variables": None, "get": content},
            status_code=200,
            content=content,
            view_excludes="sensitive_variables")
    
    def test_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        and that sensitive variables are set on the generated view.
        
        """
        self.assertViewBehavior(
            status_code=405,
            view_exact={"sensitive_variables": "__ALL__"})
    
    def test_disabled_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        and that sensitive variables are not set on the generated view when
        the behavior is disabled.
        
        """
        self.assertViewBehavior(
            {"sensitive_variables": None},
            status_code=405,
            view_excludes="sensitive_variables")


class SensitivePostParametersTestCase(base.TestCase):
    """
    Tests for the SensitivePostParameters view behavior.
    
    """
    view_classes = debug.SensitivePostParameters
    
    def test_defaults(self):
        """
        Check that the special marker for all sensitive POST parameters is set
        on the request.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"get": content},
            status_code=200,
            content=content,
            request_exact={"sensitive_post_parameters": "__ALL__"})
    
    def test_single(self):
        """
        Check that a single sensitive POST parameter is set on the
        generated request.
        
        """
        content = self.unique()
        parameter = self.unique_variable()
        self.assertViewBehavior(
            {"sensitive_post_parameters": parameter, "get": content},
            status_code=200,
            content=content,
            request_exact={"sensitive_post_parameters": (parameter,)})
    
    def test_multiple(self):
        """
        Check that multiple sensitive POST parameters are set on the request.
        
        """
        content = self.unique()
        parameters = (self.unique_variable(), self.unique_variable(),)
        self.assertViewBehavior(
            {"sensitive_post_parameters": parameters, "get": content},
            status_code=200,
            content=content,
            request_exact={"sensitive_post_parameters": parameters})
    
    def test_disabled(self):
        """
        Check that sensitive POST parameters are not set on the request when
        the behavior is disabled.
        
        """
        content = self.unique()
        self.assertViewBehavior(
            {"sensitive_post_parameters": None, "get": content},
            status_code=200,
            content=content,
            request_excludes="sensitive_post_parameters")
    
    def test_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        and that sensitive POST parameters are set on the request.
        
        """
        self.assertViewBehavior(
            status_code=405,
            request_exact={"sensitive_post_parameters": "__ALL__"})
    
    def test_disabled_precedence(self):
        """
        Check that the default HTTP method name protection takes precedence
        and that sensitive POST parameters are not set on the request when the
        behavior is disabled.
        
        """
        self.assertViewBehavior(
            {"sensitive_post_parameters": None},
            status_code=405,
            request_excludes="sensitive_post_parameters")



from __future__ import unicode_literals

from .. import redirects
from . import base


class TestCase(redirects.TestCase, base.TestCase):
    """
    A test case for making assertions about redirects for views.
    
    """
    # Assertions.
    def assertRedirects(self, *args, **kwargs):
        """
        Patches redirect assertion to ensure that the client's base get() method
        and original request handler are used.
        
        """
        with self.client.base_implementation:
            return super(TestCase, self).assertRedirects(*args, **kwargs)



from __future__ import unicode_literals

from django.utils.decorators import classonlymethod
from django.views.decorators import gzip

from .. import generic


__all__ = ("GZipPage",)


class GZipPage(generic.View):
    """
    A view behavior that gzips the response's content at the top level of
    the dispatch.
    
    Set the gzip_page attribute to a falsy value to disable the behavior.
    
    See the django.views.decorators.gzip.gzip_page() for details.
    
    """
    gzip_page = True
    
    @classonlymethod
    def as_view(cls, **kwargs):
        """
        Optionally decorates the base view with
        django.views.decorators.gzip.gzip_page().
        
        """
        view = super(GZipPage, cls).as_view(**kwargs)
        return gzip.gzip_page(view) if cls.gzip_page else view



from __future__ import unicode_literals

from django.core.cache import cache

from daydreamer.tests.views.core import http


class TestCase(http.TestCase):
    """
    Common utilities for testing cache view behaviors.
    
    """
    def setUp(self):
        cache.clear()
        super(TestCase, self).setUp()
    
    def tearDown(self):
        super(TestCase, self).tearDown()
        cache.clear()



from __future__ import unicode_literals

from django.views import generic

from .base import View


__all__ = ("FormView", "CreateView", "UpdateView", "DeleteView",)


class FormView(generic.FormView, View):
    """
    Extends Django's FormView class with features from
    daydreamer.views.generic.View.
    
    """
    pass


class CreateView(generic.CreateView, View):
    """
    Extends Django's CreateView class with features from
    daydreamer.views.generic.View.
    
    """
    pass


class UpdateView(generic.UpdateView, View):
    """
    Extends Django's UpdateView class with features from
    daydreamer.views.generic.View.
    
    """
    pass


class DeleteView(generic.DeleteView, View):
    """
    Extends Django's DeleteView class with features from
    daydreamer.views.generic.View.
    
    """
    pass

